
const elasticsearch = require('elasticsearch');
const fs = require('fs');
const config = require('../config/elasticsearch');

const defaultUrl = `${config.analytics.host}:${config.analytics.port}`;

let self = {};
function ElasticSearch() {
  self = new elasticsearch.Client({
    host:
      process.env.NODE_ENV === 'production'
        ? `https://${process.env.OM_ELASTIC_AUTH}@${defaultUrl}` : defaultUrl,
    log: 'error',
    maxSockets: 40,
  });

  self.ping({ requestTimeout: 30000 }, (error) => {
    if (error) {
      console.error('ElasticSearch->DOWN!');
    } else {
      console.log('ElasticSearch->OK');
    }
  });
}

function dropIndex(indexName) {
  return self.indices.delete({
    index: indexName,
  });
}

function indexExists(indexName) {
  return self.indices.exists({
    index: indexName,
  });
}

function checkIndex(indexName) {
  return this.indexExists(indexName).then((exists) => {
    if (!exists) {
      return self.indices.create({
        index: indexName,
      });
    }
    return true;
  });
}

function save(indexName, body) {
  if (typeof body === 'object' && body.time === undefined) {
    body.time = new Date();
    body.doctype = indexName;
  }
  return self.index({
    index: indexName,
    type: 'document',
    body,
  });
}

function searchByQuery(indexName, query) {
  return self.search({
    index: indexName,
    q: query,
  });
}

function searchByBody(indexName, body, size) {
  const options = {
    index: indexName,
    body,
  };
  if (size !== undefined) {
    options.size = size;
  }
  return self.search(options);
}

function countByBody(indexName, body) {
  const options = {
    index: indexName,
    body,
  };
  return self.count(options);
}

function getFromIndex(indexName, id) {
  return self.get({
    index: indexName,
    type: 'document',
    id,
  });
}

function waitForIndexing() {
  return new Promise((resolve) => {
    setTimeout(resolve, 2000);
  });
}

function checkIndices() {
  return self.cat.indices(
    { v: true },
  );
}

function indiceMapping(indexName, type, bodySchema) {
  return self.indices.putMapping({
    index: indexName,
    type,
    body: bodySchema,
  });
}

function bulkIndex(index, data) {
  const bulkBody = [];
  data.map((item) => {
    bulkBody.push({
      index: {
        _index: index,
        _type: 'document',
      },
    });
    item.time = new Date();
    bulkBody.push(item);
  });

  self.bulk({ body: bulkBody })
    .then((response) => {
      let errorCount = 0;
      response.items.forEach((item) => {
        if (item.index && item.index.error) {
          errorCount += 1;
          console.log(errorCount, item.index.error);
        }
      });
      console.log(
        `Successfully indexed ${data.length - errorCount} out of ${data.length} items`,
      );
    }).catch(console.err);
}

async function indexFromJSON(indexName, type, fileName) {
  const sampleJson = await fs.readFileSync(`./${fileName}`);
  this.bulkIndex(indexName, type, sampleJson);
}

function closeConnection() {
  self.close();
}

module.exports = ElasticSearch;
